@@ -8,7 +8,9 @@ import functools
 import io
 import logging
 import os
+import warnings
 
+from django.contrib.auth.hashers import make_password
 from django.core import files as django_files
 from django.db import IntegrityError
 
@@ -47,6 +49,7 @@ class DjangoOptions(base.FactoryOptions):
         return super()._build_default_options() + [
             base.OptionDefault('django_get_or_create', (), inherit=True),
             base.OptionDefault('database', DEFAULT_DB_ALIAS, inherit=True),
+            base.OptionDefault('skip_postgeneration_save', False, inherit=True),
         ]
 
     def _get_counter_reference(self):
@@ -79,6 +82,7 @@ class DjangoModelFactory(base.Factory):
     """
 
     _options_class = DjangoOptions
+    _original_params = None
 
     class Meta:
         abstract = True  # Optional, but explicit.
@@ -139,6 +143,10 @@ class DjangoModelFactory(base.Factory):
         try:
             instance, _created = manager.get_or_create(*args, **key_fields)
         except IntegrityError as e:
+
+            if cls._original_params is None:
+                raise e
+
             get_or_create_params = {
                 lookup: value
                 for lookup, value in cls._original_params.items()
@@ -165,14 +173,29 @@ class DjangoModelFactory(base.Factory):
         manager = cls._get_manager(model_class)
         return manager.create(*args, **kwargs)
 
+    # DEPRECATED. Remove this override with the next major release.
     @classmethod
     def _after_postgeneration(cls, instance, create, results=None):
         """Save again the instance if creating and at least one hook ran."""
-        if create and results:
+        if create and results and not cls._meta.skip_postgeneration_save:
+            warnings.warn(
+                f"{cls.__name__}._after_postgeneration will stop saving the instance "
+                "after postgeneration hooks in the next major release.\n"
+                "If the save call is extraneous, set skip_postgeneration_save=True "
+                f"in the {cls.__name__}.Meta.\n"
+                "To keep saving the instance, move the save call to your "
+                "postgeneration hooks or override _after_postgeneration.",
+                DeprecationWarning,
+            )
             # Some post-generation hooks ran, and may have modified us.
             instance.save()
 
 
+class Password(declarations.Transformer):
+    def __init__(self, password, transform=make_password, **kwargs):
+        super().__init__(password, transform=transform, **kwargs)
+
+
 class FileField(declarations.BaseDeclaration):
     """Helper to fill in django.db.models.FileField from a Factory."""
 
@@ -185,25 +208,28 @@ class FileField(declarations.BaseDeclaration):
     def _make_content(self, params):
         path = ''
 
-        _content_params = [params.get('from_path'), params.get('from_file'), params.get('from_func')]
-        if len([p for p in _content_params if p]) > 1:
+        from_path = params.get('from_path')
+        from_file = params.get('from_file')
+        from_func = params.get('from_func')
+
+        if len([p for p in (from_path, from_file, from_func) if p]) > 1:
             raise ValueError(
                 "At most one argument from 'from_file', 'from_path', and 'from_func' should "
                 "be non-empty when calling factory.django.FileField."
             )
 
-        if params.get('from_path'):
-            path = params['from_path']
+        if from_path:
+            path = from_path
             with open(path, 'rb') as f:
                 content = django_files.base.ContentFile(f.read())
 
-        elif params.get('from_file'):
-            f = params['from_file']
+        elif from_file:
+            f = from_file
             content = django_files.File(f)
             path = content.name
 
-        elif params.get('from_func'):
-            func = params['from_func']
+        elif from_func:
+            func = from_func
             content = django_files.File(func())
             path = content.name
 
@@ -301,6 +327,9 @@ class mute_signals:
             # Retrieve __func__, the *actual* callable object.
             callable_obj._create = self.wrap_method(callable_obj._create.__func__)
             callable_obj._generate = self.wrap_method(callable_obj._generate.__func__)
+            callable_obj._after_postgeneration = self.wrap_method(
+                callable_obj._after_postgeneration.__func__
+            )
             return callable_obj
 
         else:
